#!/usr/bin/env perl
#
# PLAYGROUND.PL - generate a random filled tofu stack.
#
#

# Copyright (c) 2008 Sbastien Boillod <sbb at tuxfamily dot org>.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
use warnings;
use strict;
use File::Path;

sub help {
    # Print the help and exit.
    # help
    my $s = " " x 4;
    print "playground.pl - stack generator for tofu.\n\n";
    print "usage: playground.pl <name|-number>\n\n";
    print "$s-<n>        generate a stack with <n> random filled tasks inside.\n";
    print "$s<name>      name the stack <name>.\n";
    print "$s-root=<dir> use <dir> instead of '~/.tofu'.\n";
    print "$s-tutorial   explicit tutorial mode ('play' is the default stack name).\n\n";
    exit 0;
}

sub hello {
    # Parse the arguments on the command-line, then return the stack
    # type to create and its name.
    # hello
    my ($p, $r, $n)  = ("$ENV{HOME}/.tofu/", 0, "play");
    &help unless @ARGV;
    for (@ARGV) {
        if (m/^-([1-9][0-9]*)$/) {
            $r = $1;
        } elsif (m/^-root=(.+)$/) {
            $p = $1;
        } elsif (m/^-tutorial$/) {
            $r = 0;
        } elsif (m/^-/) {
            die "Unallowed '$_' option.\n";
        } elsif (not m/^\/*$/) { # prevent from removing ~/.tofu/ with "/" argument.
            $n = $_;
        }
    }
    return ("$p/$n", $r, $n);
}


sub rmarkers {
    # Return a random list of markers
    # rmarkers
    my %mrk = ("one", 1, "two", 2, "three", 3, "four", 4);
    my @list = ();
    for (sort (keys %mrk)) {
        push @list, $_ unless int(rand 9999) % $mrk{$_};
    }
    return "@list";
}

sub rpriority {
    # Return a random priority.
    # rpriority
    my $pr = "COOL";
    if (not int(rand 9999)%3) {
        $pr = "HOT";
    } elsif (not int(rand 9999)%2) {
        $pr = "WARM";
    }
    return $pr
}


my ($path, $rdm, $stk) = &hello(@ARGV);
# We initialize the stack
rmtree($path);
mkpath($path, 0, 0755);

# We make the stack.
open STACK, ">$path/stack" or die "Can't create the '$path/stack' file.\n";
open TEXT, ">$path/content" or die "Can't create the '$path/content' file.\n";
if ($rdm) {
    # Random mode.
    print "Generate the '$stk' stack ($rdm tasks, random mode).\n";
    print STACK ">" . ($rdm + 1) . ",0\n";
    while ($rdm) {
        print STACK "$rdm " . &rpriority . " " . &rmarkers . "\n";
        print TEXT "$rdm>Have a lot of fun in the playground\n$rdm>\n$rdm>\n" .
                   "$rdm>    Just a little castle made of Perl ^_^\n$rdm>\n" . $rdm-- . ">\n";
    }
} else {
    # Tutorial mode.
    print "Generate the '$stk' stack (tutorial mode).\n";
    my @tsk = ("COOL one three", "HOT two", "WARM three", "COOL four two", "COOL one three",
               "COOL", "WARM two three", "HOT one two", "COOL one", "WARM four one");
    for (@tsk) {
        print STACK ++$rdm . " " . $_ ."\n";
        print TEXT "$rdm>Have a lot of fun in the playground\n$rdm>\n$rdm>\n" .
                   "$rdm>    Just a little castle made of Perl ^_^\n$rdm>\n$rdm>\n";
    }
    print STACK ">" . ++$rdm . ",0\n";
}
close STACK; close TEXT;

# EoF
